--------------------------------------------------------------------------------
-- Company: Trenz Electronic GmbH
-- Engineer: Antti Lukats, John Hartfiel
--
-- TE0703CPLD U5 
--
--
-- Revision     : 02
-- Release Date : 2017-10-13
-- Supported PCB Revision: 02,03,04,05
--
-- Revision chances to CC703_simple:
-- - remove unused logic and IOs
-- - add MIO9 SD_SEL,SD_CD, NOSEQ, PGOOD, 2LEDs, PHY LEDs
-- - debounce button

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;

library machxo2;
use machxo2.all;

ENTITY top IS
  PORT (
    -- FTDI Channel A, used normally as JTAG/MPSSE
	M_TCK         		: IN std_logic;   -- ADBUS0
	M_TDI         		: IN std_logic;   -- ADBUS1
	M_TDO         		: OUT std_logic;  -- ADBUS2
	M_TMS         		: IN std_logic;   -- ADBUS3

	BDBUS0              : IN std_logic; -- TX from FTDI
	BDBUS1              : OUT std_logic; -- RX to FTDI	

	-- PS MIO on TE0720 or 3.3V FPGA I/O on TE0710/TE0741 modules
	MIO0                : IN std_logic; --	 	
  MIO9                : OUT std_logic; -- use as SD Card Detect
	
	-- MIO10               : IN std_logic; --	 	
	-- MIO11               : IN std_logic; --
	
	MIO12               : IN std_logic; -- 
	MIO13               : OUT std_logic; --
	
	MIO14               : OUT std_logic; -- UART0.RX << BDBUS0
	MIO15               : IN std_logic; -- UART0.TX >> BDBUS1

	-- X                   : INOUT std_logic_vector(15 downto 0);
	X17         		: OUT std_logic;
	X16         		: IN std_logic;

	-- Module JTAG
	C_TMS         		: OUT std_logic;
	C_TCK         		: OUT std_logic;
	C_TDI         		: OUT std_logic;
	C_TDO         		: IN std_logic;

	-- Module special pins
	PROGMODE      		: OUT std_logic;
	RESIN      		    : OUT std_logic;
	NOSEQ      		    : INOUT std_logic;
	PGOOD     		    : INOUT std_logic;
	MODE      		    : OUT std_logic;
	EN1      		      : OUT std_logic;

	-- Carrier USER I/O, etc
	S1           		: IN std_logic; -- Push Button (usable as external reset)

	-- SD :
  SD_SEL          : out std_logic;
  SD_CD           : in std_logic;
	-- DIP switches :
	CM0         		: IN std_logic; -- Mode Select, note powered only when module is inserted and/or VCCIO powered
	CM1         		: IN std_logic; --disable SD_CD forwarding for older REV02,03,04 PCB Versions
    -- No CM2 or CM3
  --LED
	LED1                : OUT std_logic;    --red D1 
	LED2                : OUT std_logic;    --green D2
	FL_0                : INOUT std_logic;  --red D3
	FL_1                : INOUT std_logic;  --green D4
	PHY_LED1            : OUT std_logic;   --
	PHY_LED1R           : OUT std_logic;   --
	PHY_LED2            : OUT std_logic;   --
	PHY_LED2R           : OUT std_logic;   --
  --dummy pin
	dummy               : OUT std_logic   --Pin 35
);
END ENTITY top;

ARCHITECTURE translated OF top IS

 -- onchip oscillator
 COMPONENT OSCH
	-- synthesis translate_off
	GENERIC (NOM_FREQ: string := "12.09");
	-- synthesis translate_on
	PORT(	STDBY :IN std_logic;
			OSC :OUT std_logic;
			SEDSTDBY :OUT std_logic);
END COMPONENT OSCH;
attribute NOM_FREQ : string;
attribute NOM_FREQ of OSCinst0 : label is "12.09";
-----------------------------------
constant DB_MAX : integer := 20;

signal FTDI_RXD : std_logic; -- (alias) data from FTDI 			
signal FTDI_TXD : std_logic; -- (alias) data from (to??)

signal Clk: std_logic;

signal  cnt_i : std_logic_vector(31 downto 0) := (others => '0');

signal  c_rst_db_cnt : std_logic_vector(DB_MAX downto 0) := (others => '0');
signal  n_rst_db_cnt : std_logic_vector(DB_MAX downto 0) := (others => '0');
signal  button_rst_db : std_logic_vector(3 downto 0) := (others =>'1');
signal  db_rst_rst : std_logic;
signal  rst_btn_i				:std_logic;

BEGIN

------------------------
--debounce rst button
  button_rst_db(0) <= S1;
  db_2_proc: process(Clk)
  begin
    if(rising_edge(Clk))then
      button_rst_db(2 downto 1)		<= button_rst_db(1 downto 0);
      if(c_rst_db_cnt(DB_MAX)='1') then
        button_rst_db(3) <= button_rst_db(2);
      end if;
    end if;
  end process;
  db_rst_rst <= button_rst_db(2) xor button_rst_db(1);

  db_2_cnt_proc: process(Clk,db_rst_rst)
  begin
    if(db_rst_rst='1') then
      c_rst_db_cnt <= (others =>'0');
    else
      if rising_edge(Clk) then
        c_rst_db_cnt 	<= n_rst_db_cnt;
      end if;
    end if;
  end process;
  n_rst_db_cnt <= c_rst_db_cnt + '1' when c_rst_db_cnt(DB_MAX)='0' else c_rst_db_cnt;
  rst_btn_i <= button_rst_db(3);


-------------------------------------	
dummy <= NOSEQ or PGOOD;
-------------------------------------	
-- Clock source :
  OSCInst0: OSCH
	-- synthesis translate_off
	GENERIC MAP ( NOM_FREQ => "12.09" )
	-- synthesis translate_on
	PORT MAP (STDBY=> '0', OSC=> Clk, SEDSTDBY=> open);
-------------------------------------	
-- counter
  cnt_proc: process(Clk)
  begin
    if rising_edge(Clk) then
      cnt_i 	<= cnt_i + '1';
    end if;
  end process;

-------------------------------------	
-- JTAG Chain multiplexer : just pass through...
  C_TMS <= M_TMS;-- when CR3(0)='0' else 'Z';   -- X15
  C_TCK <= M_TCK;-- when CR3(0)='0' else 'Z';   -- X14
  C_TDI <= M_TDI;-- when CR3(0)='0' else 'Z';   -- X13
  M_TDO <= C_TDO;-- when CR3(0)='0' else X(12); -- X12
  
-------------------------------------	
--sequencing
	NOSEQ <= 'Z';
	PGOOD <= 'Z';
-------------------------------------	
--power
	EN1 	<= '1';
-------------------------------------	
--Reset
	RESIN	<= rst_btn_i; -- Manual Module reset
-------------------------------------	
--sd
  SD_SEL <= '0';
  MIO9 <= SD_CD and CM1;
-------------------------------------	
-- Boot mode
	MODE  	<= MIO0; --; -- Select SD Boot Mode... must be HARD WIRED!
-------------------------------------	
--Module JTAG Selection  
	PROGMODE <= CM0; -- CM0 developer Mode, uses Switch to control JTAGSEL pin in B2B Connector
-------------------------------------	
--LED
	LED1 	<= cnt_i(24)  when PGOOD='0' else '1' when CM0='1' else not FTDI_RXD; --red D1 
	LED2 	<= cnt_i(24)  when PGOOD='0' else '1' when CM0='1' else not FTDI_TXD; --green D2
                                               
  FL_0 <= cnt_i(24)  when PGOOD='0' else '1' when CM0='1' else 'Z';  --red D3
	FL_1 <= cnt_i(24)  when PGOOD='0' else '1' when CM0='1' else 'Z';  --green D4
  
  -- slow blink (yellow green), if Module CPLD Sel pin is selected
  -- fast blink (yellow),       if Module Power Problem occures
  -- LED Green,      QSPI Boot
  -- LED Orange,      SD Boot
	PHY_LED1    <= '0'        when PGOOD='0' else cnt_i(24)      when CM0='1' else '1' when MIO0='1' else  '0';
	PHY_LED1R   <= cnt_i(22)  when PGOOD='0' else not cnt_i(24)  when CM0='1' else '0' when MIO0='1' else  '1';
	PHY_LED2    <= cnt_i(22)  when PGOOD='0' else cnt_i(24)      when CM0='1' else '0' when MIO0='1' else  '1';
	PHY_LED2R   <= '0'        when PGOOD='0' else not cnt_i(24)  when CM0='1' else '1' when MIO0='1' else  '0';
  
	
-------------------------------------	
-- Connect Module MIO14,15 (Zynq PS UART) to FTDI Channel B UART
  --primary uart
	FTDI_RXD  <= BDBUS0;
	BDBUS1	  <= FTDI_TXD;
	MIO14	    <= FTDI_RXD;
	FTDI_TXD  <= MIO15;
	
  --secondary uart
	X17		 <= MIO12;
	MIO13	 <= X16;	



END ARCHITECTURE translated;